//--------LICENSE_START--------
//
// Copyright (C) 2020,2021,2022 Institut de Robòtica i Informàtica Industrial, CSIC-UPC.
// Authors: Joan Solà Ortega (jsola@iri.upc.edu)
// All rights reserved.
//
// This file is part of WOLF
// WOLF is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//--------LICENSE_END--------
#ifndef CAPTURE_GNSS_FIX_H_
#define CAPTURE_GNSS_FIX_H_

//Wolf includes
#include "gnss/feature/feature_gnss_fix.h"
#include "core/capture/capture_base.h"

//std includes
//

namespace wolf {
    
WOLF_PTR_TYPEDEFS(CaptureGnssFix);

//class CaptureGnssFix
class CaptureGnssFix : public CaptureBase
{
    protected:
        Eigen::Vector3d position_ECEF_; ///< position in ECEF coordinates.
        Eigen::Matrix3d covariance_ECEF_; ///< Noise of the fix in ECEF coordinates.
        TimeStamp ts_GPST_; ///< Time stamp in GPS time

    public:
        CaptureGnssFix(const TimeStamp& _ts, SensorBasePtr _sensor_ptr, const Eigen::Vector3d& _position, const Eigen::Matrix3d& _covariance, bool _ecef_coordinates = true);
        ~CaptureGnssFix() override;

        const Eigen::Vector3d& getPositionEcef() const;
        const Eigen::Matrix3d& getCovarianceEcef() const;
        void getPositionAndCovarianceEcef(Eigen::Vector3d& position, Eigen::Matrix3d& cov) const;
        const TimeStamp& getGpsTimeStamp() const;
        void setGpsTimeStamp(const TimeStamp& _ts_GPST);
};

inline const Eigen::Vector3d& CaptureGnssFix::getPositionEcef() const
{
    return position_ECEF_;
}

inline const Eigen::Matrix3d& CaptureGnssFix::getCovarianceEcef() const
{
    return covariance_ECEF_;
}

inline void CaptureGnssFix::getPositionAndCovarianceEcef(Eigen::Vector3d& position, Eigen::Matrix3d& cov) const
{
    position = position_ECEF_;
    cov = covariance_ECEF_;
}

inline const wolf::TimeStamp& CaptureGnssFix::getGpsTimeStamp() const
{
    return ts_GPST_;
}

inline void CaptureGnssFix::setGpsTimeStamp(const TimeStamp& _ts_GPST)
{
    ts_GPST_ = _ts_GPST;
}

} //namespace wolf
#endif
